;  2000 Peter N Lewis <peter@stairways.com.au>
; This code is public domain and may be used freely for any purpose

.include "TinyDEF.INC"
.cseg
.org 0

; Calling conventions:
; parameters (in/out) passed in r20, r21, ...
; parameters are not preserved
; r16-r19 are not preserved (except interrupt routines which start with i)
; all other registers are preserved

; Warning - minute is really 15 seconds!

.def	rkM1	= r1	; -1
.def	rk0	= r2	; 0
.def	rk1	= r3	; 1
.def	rk2	= r4	; 2
.def	rkLedOn	= r5

.def	rFlashTimer = r21
.def	rMinuteL	= r22
.def	rMintueH	= r23
.def	rCycleL	= r24
.def	rCycleH	= r25

.equ	kLEDOn	= $04
.equ	kBuzOn	= $08
.equ	kOutputs	= kLEDOn + kBuzOn
.equ	kHi	= $10
.equ	kLo	= $02

.equ	kCycles	= 6729; 27000; 27466; 31912; 33584
.equ	k23HourMinutes = 5800; 240; 8; 1380
.equ	kBuzMinutes = 20
.equ	kFlashPeriod = 125
.equ	kQuickFlashPeriod = 50

.org 0
vRESET:	rjmp	RESET
vINT0:	rjmp	DIE
vOVF0:	rjmp	DIE	

RESET:
	cli
	
	clr	rkM1
	dec	rkM1
	clr	rk0
	clr	rk1
	inc	rk1
	mov	rk2, rk1
	inc	rk2
	
	ldi	r16, kLEDOn
	mov	rkLedOn, r16
	
	ldi	r16, XRAMEND
	out	SPL, r16
	
	ldi	r16, $1E
	out	DDRB, r16
	
	ldi	r16, kLo + kLEDOn
	out	PORTB, r16
	
	clr	rFlashTimer

	ldi	rMinuteL, low(k23HourMinutes)
	ldi	rMintueH, high(k23HourMinutes)
	
WMinute:	ldi	rCycleL, low(kCycles)
	ldi	rCycleH, high(kCycles)
	
WLoop:	rcall	WaitCycle

	tst	rFlashTimer
	breq	WNoFlash
	
	dec	rFlashTimer
	brne	WNoFlash

	in	r16, PORTB
	sbr	r16, kLedOn
	out	PORTB, r16

WNoFlash:
	sub	rCycleL, rk1
	sbc	rCycleH, rk0
	
	mov	r16, rCycleL
	or	r16, rCycleH
	
	brne	WLoop

; Oncer per minute 
	ldi	rFlashTimer, kQuickFlashPeriod
	in	r16, PORTB
	cbr	r16, kLedOn
	out	PORTB, r16

	sub	rMinuteL, rk1
	sbc	rMintueH, rk0
	
	mov	r16, rMinuteL
	or	r16, rMintueH
	
	brne	WMinute

; After 23 hours

Alert: 	mov	rFlashTimer, rk1

NextBuz:
	in	r16, PORTB
	sbr	r16, kBuzOn
	out	PORTB, r16
	cbr	r16, kBuzOn
	out	PORTB, r16

	ldi	rMinuteL, low(kBuzMinutes)
	ldi	rMintueH, high(kBuzMinutes)
	
AMinute:	ldi	rCycleL, low(kCycles)
	ldi	rCycleH, high(kCycles)
	
ALoop:	rcall	WaitCycle
	
	dec	rFlashTimer
	brne	NoFlash
	
	ldi	rFlashTimer, kFlashPeriod
	
	in	r16, PORTB
	eor	r16, rkLedOn
	out	PORTB, r16
	
NoFlash:
	sub	rCycleL, rk1
	sbc	rCycleH, rk0
	
	mov	r16, rCycleL
	or	r16, rCycleH
	
	brne	ALoop

; Oncer per minute 

	sub	rMinuteL, rk1
	sbc	rMintueH, rk0
	
	mov	r16, rMinuteL
	or	r16, rMintueH
	
	brne	AMinute
	
	rjmp	NextBuz



WaitCycle:

WaitHi:	in	r16, PINB
	andi	r16, $01
	breq	WaitHi
	
	in	r16, PORTB
	andi	r16, kOutputs
	ori	r16, kHi
	out	PORTB, r16
	
	rcall	Delay20us

WaitLo:	in	r16, PINB
	andi	r16, $01
	brne	WaitLo
	
	in	r16, PORTB
	andi	r16, kOutputs
	ori	r16, kLo
	out	PORTB, r16
	
	rcall	Delay20us
	
	ret


DIE:	ldi	r16, kLEDOn
	out	DDRB, r16
	out	PORTB, r16
	ldi	r20, 1
	rcall	DELSEC
	ldi	r16, $00
	out	PORTB, r16
	ldi	r20, 1
	rcall	DELSEC
	rjmp	DIE

Delay20us:
	push	r16
	ldi	r16, 5
D1Lop:	dec	r16
	brne	D1Lop
	pop	r16
	ret


	
; Delay r20 seconds
DELSEC:	push	r31
	mov	r31, r20	; Stupid reversed moves, r31 <= r20
DS1:	ldi	r20, 60
	rcall	DELAYTicks
	dec	r31
	brne	DS1
	pop	r31
	ret
	
.equ	DELTICKS = 6 ; 65

; Delay r20 ticks (1/60th second)
DELAYTicks:
DL0:	ldi	r16, DELTICKS
DL1:	ldi	r17, 0
DL2:	rcall	DUMMY
	dec	r17
	brne	DL2
	dec	r16
	brne	DL1
	dec	r20
	brne	DL0
DUMMY:	ret

LongDelayLCD:
	push	r17
	push	r20
	ldi	r20, 40
	rcall	DELAY50
	pop	r20
	pop	r17
	ret

DelayLCD:
	push	r17
	push	r20
	ldi	r20, 1
	rcall	DELAY50
	pop	r20
	pop	r17
	ret

; Delay r20 * 50uS
DELAY50:
DL501:	ldi	r17, 51
DL502:	rcall	DUMMY50
	dec	r17
	brne	DL502
	dec	r20
	brne	DL501
DUMMY50:
	ret
